@@ -17,6 +17,7 @@
 #endif
 
 #include <inttypes.h>
+#include "../src/xine-utils/attributes.h"
 
 
 #if defined(WIN32) || defined(__CYGWIN__)
@@ -267,4 +268,13 @@ void          xine_private_rewinddir(DIR *);
 
 #endif
 
+/* replacement of memmem */
+#ifndef HAVE_MEMMEM
+#define HAVE_MEMMEM
+#define memmem(H, HL, N, NL) \
+  xine_private_memmem((H), (HL), (N), (NL))
+void *xine_private_memmem (const void *haystack, size_t haystack_len,
+			   const void *needle, size_t needle_len);
+#endif
+
 #endif
