@@ -15,12 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /*
- * $Id: demux_avi.c,v 1.232 2007/03/29 19:24:18 dgp85 Exp $
- *
  * demultiplexer for avi streams
  *
  * part of the code is taken from
@@ -46,7 +44,6 @@
  * expect to find the next A/V frame.  We periodically check if we can
  * read data from the file at that offset.  If we can, we append index
  * data for as many frames as we can read at the time.
- *
  */
 
 /*
@@ -55,7 +52,6 @@
  * Transcode's and xine's avi code comes from the same source and
  * still has a very similar architecture, so it wasn't much effort to
  * port it from transcode to xine.
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -953,7 +949,7 @@ static avi_t *AVI_init(demux_avi_t *this) {
 
         lprintf ("palette_count: %d\n", AVI->palette_count);
         if (AVI->palette_count > 256) {
-          lprintf ("number of colors exceeded 256 (%d)", AVI->palette_count);
+          lprintf ("number of colours exceeded 256 (%d)", AVI->palette_count);
           AVI->palette_count = 256;
         }
         if ((strf_size - sizeof(xine_bmiheader)) >= (AVI->palette_count * 4)) {
@@ -977,6 +973,10 @@ static avi_t *AVI_init(demux_avi_t *this) {
         xine_waveformatex *wavex;
         
         wavex = (xine_waveformatex *)malloc(n);
+        if (!wavex) {
+          this->AVI_errno = AVI_ERR_NO_MEM;
+          return 0;
+        }
         memcpy((void *)wavex, hdrl_data+i, n);
         _x_waveformatex_le2me( wavex );
 
@@ -1241,6 +1241,9 @@ static avi_t *AVI_init(demux_avi_t *this) {
           /* read from file */
           chunk_start = en = malloc (AVI->video_superindex->aIndex[j].dwSize+hdrl_len);
 
+          if (!chunk_start)
+             ERR_EXIT(AVI_ERR_NO_MEM);
+
           if (this->input->seek(this->input, AVI->video_superindex->aIndex[j].qwOffset, SEEK_SET) == (off_t)-1) {
              lprintf("cannot seek to 0x%" PRIx64 "\n", AVI->video_superindex->aIndex[j].qwOffset);
              free(chunk_start);
@@ -1312,6 +1315,9 @@ static avi_t *AVI_init(demux_avi_t *this) {
           /* read from file */
           chunk_start = en = malloc (audio->audio_superindex->aIndex[j].dwSize+hdrl_len);
 
+          if (!chunk_start)
+            ERR_EXIT(AVI_ERR_NO_MEM);
+
           if (this->input->seek(this->input, audio->audio_superindex->aIndex[j].qwOffset, SEEK_SET) == (off_t)-1) {
             lprintf("cannot seek to 0x%" PRIx64 "\n", audio->audio_superindex->aIndex[j].qwOffset);
             free(chunk_start);
