@@ -15,10 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: xine_decoder.c,v 1.49 2007/02/20 00:34:57 dgp85 Exp $
- *
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 #include <stdlib.h>
@@ -197,6 +194,8 @@ static int faad_open_output( faad_decoder_t *this ) {
     case 2:
       this->ao_cap_mode=AO_CAP_MODE_STEREO;
       break; 
+  default:
+    return 0;
   }
    
   this->output_open = (this->stream->audio_out->open) (this->stream->audio_out,
