@@ -18,9 +18,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: cc_decoder.c,v 1.28 2007/02/20 00:56:36 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * stuff needed to provide closed captioning decoding and display
  *
@@ -223,10 +221,14 @@ static const uint8_t *const cc_alpha_palettes[NUM_CC_PALETTES] = {
 /* mapping from PAC row code to actual CC row */
 static const int  rowdata[] = {10, -1, 0, 1, 2, 3, 11, 12, 13, 14, 4, 5, 6,
 			 7, 8, 9};
-/* FIXME: do real TM */
-/* must be mapped as a music note in the captioning font */ 
-static const char specialchar[] = {'','','','','T','','','','',
-			     TRANSP_SPACE,'','','','','',''};
+/* FIXME: do real ™ (U+2122) */
+/* Code 182 must be mapped as a musical note ('♪', U+266A) in the caption font */ 
+static const char specialchar[] = {
+  174 /* ® */, 176 /* ° */, 189 /* ½ */, 191 /* ¿ */,
+  'T' /* ™ */, 162 /* ¢ */, 163 /* £ */, 182 /* ¶ => ♪ */,
+  224 /* à */, TRANSP_SPACE,232 /* è */, 226 /* â */,
+  234 /* ê */, 238 /* î */, 244 /* ô */, 251 /* û */
+};
 
 /* character translation table - EIA 608 codes are not all the same as ASCII */
 static char chartbl[128];
@@ -427,16 +429,16 @@ static void build_char_table(void)
   for (i = 0; i < 128; i++)
     chartbl[i] = (char) i;
   /* now the special codes */
-  chartbl[0x2a] = '';
-  chartbl[0x5c] = '';
-  chartbl[0x5e] = '';
-  chartbl[0x5f] = '';
-  chartbl[0x60] = '';
-  chartbl[0x7b] = '';
-  chartbl[0x7c] = '';
-  chartbl[0x7d] = '';
-  chartbl[0x7e] = '';
-  chartbl[0x7f] = '';    /* FIXME: this should be a solid block */
+  chartbl[0x2a] = 225; /* á */
+  chartbl[0x5c] = 233; /* é */
+  chartbl[0x5e] = 237; /* í */
+  chartbl[0x5f] = 243; /* ó */
+  chartbl[0x60] = 250; /* ú */
+  chartbl[0x7b] = 231; /* ç */
+  chartbl[0x7c] = 247; /* ÷ */
+  chartbl[0x7d] = 209; /* Ñ */
+  chartbl[0x7e] = 241; /* ñ */
+  chartbl[0x7f] = 164; /* ¤ FIXME: should be a solid block ('█'; U+2588) */
 }
 
 
