@@ -15,8 +15,8 @@
 #include "vfw.h"
 #include "registry.h"
 #include "ldt_keeper.h"
-
-extern char* win32_def_path;
+#include "ext.h"
+#include "debugtools.h"
 
 
 #if 1
@@ -52,6 +52,7 @@ extern char* win32_def_path;
     "pop %%ebx\n\t"::)
 #endif
 
+#if 0
 static int needs_free=0;
 void SetCodecPath(const char* path)
 {
@@ -66,6 +67,7 @@ void SetCodecPath(const char* path)
     strcpy(win32_def_path, path);
     needs_free=1;
 }
+#endif
 
 static DWORD dwDrvID = 0;
 
@@ -85,7 +87,9 @@ LRESULT WINAPI SendDriverMessage(HDRVR hDriver, UINT message,
     __asm__ __volatile__ ("fsave (%0)\n\t": :"r"(&qw));
 #endif
 
+#ifdef LDT_paranoia
     Setup_FS_Segment();
+#endif
 
     STORE_ALL;
     result=module->DriverProc(module->dwDriverID, hDriver, message, lParam1, lParam2);
@@ -108,7 +112,9 @@ void DrvClose(HDRVR hDriver)
 	DRVR* d = (DRVR*)hDriver;
 	if (d->hDriverModule)
 	{
+#ifdef LDT_paranoia
 	    Setup_FS_Segment();
+#endif
 	    if (d->DriverProc)
 	    {
 		SendDriverMessage(hDriver, DRV_CLOSE, 0, 0);
@@ -126,11 +132,13 @@ void DrvClose(HDRVR hDriver)
 HDRVR DrvOpen(LPARAM lParam2)
 {
     NPDRVR hDriver;
-    int i;
+//    int i;
     char unknown[0x124];
     const char* filename = (const char*) ((ICOPEN*) lParam2)->pV1Reserved;
 
+#ifdef LDT_paranoia
     Setup_LDT_Keeper();
+#endif
     printf("Loading codec DLL: '%s'\n",filename);
 
     hDriver = (NPDRVR) malloc(sizeof(DRVR));
@@ -139,7 +147,9 @@ HDRVR DrvOpen(LPARAM lParam2)
     memset((void*)hDriver, 0, sizeof(DRVR));
 
     CodecAlloc();
+#ifdef LDT_paranoia
     Setup_FS_Segment();
+#endif
 
     hDriver->hDriverModule = LoadLibraryA(filename);
     if (!hDriver->hDriverModule)
@@ -158,7 +168,7 @@ HDRVR DrvOpen(LPARAM lParam2)
 	return ((HDRVR) 0);
     }
 
-    TRACE("DriverProc == %X\n", hDriver->DriverProc);
+    TRACE("DriverProc == %p\n", hDriver->DriverProc);
     SendDriverMessage((HDRVR)hDriver, DRV_LOAD, 0, 0);
     TRACE("DRV_LOAD Ok!\n");
     SendDriverMessage((HDRVR)hDriver, DRV_ENABLE, 0, 0);
@@ -167,7 +177,7 @@ HDRVR DrvOpen(LPARAM lParam2)
 
     // open driver and remmeber proper DriverID
     hDriver->dwDriverID = SendDriverMessage((HDRVR)hDriver, DRV_OPEN, (LPARAM) unknown, lParam2);
-    TRACE("DRV_OPEN Ok!(%X)\n", hDriver->dwDriverID);
+    TRACE("DRV_OPEN Ok!(%lX)\n", hDriver->dwDriverID);
 
     printf("Loaded DLL driver %s\n", filename);
     return (HDRVR)hDriver;
