@@ -15,9 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: audio_filters.h,v 1.1.2.1 2005/07/17 21:50:05 miguelfreitas Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * catalog for audio filter plugins
  */
@@ -28,3 +26,4 @@
 void *upmix_init_plugin(xine_t *xine, void *data);
 void *upmix_mono_init_plugin(xine_t *xine, void *data);
 void *stretch_init_plugin(xine_t *xine, void *data);
+void *volnorm_init_plugin(xine_t *xine, void *data);
