@@ -26,6 +26,8 @@
 #include "goom_fx.h"
 #include "goomsl.h"
 
+#include "xine_internal.h"
+
 /* #define VERBOSE */
 
 #define STOP_SPEED 128
@@ -33,18 +35,20 @@
 #define TIME_BTW_CHG 300
 
 static void choose_a_goom_line (PluginInfo *goomInfo, float *param1, float *param2, int *couleur,
-                                int *mode, float *amplitude, int far);
+                                int *mode, float *amplitude, int isfar);
 
 static void update_message (PluginInfo *goomInfo, char *message);
 
 static void init_buffers(PluginInfo *goomInfo, int buffsize)
 {
     goomInfo->pixel = (guint32 *) malloc (buffsize * sizeof (guint32) + 128);
-    bzero (goomInfo->pixel, buffsize * sizeof (guint32) + 128);
+    memset (goomInfo->pixel, 0, buffsize * sizeof (guint32) + 128);
     goomInfo->back = (guint32 *) malloc (buffsize * sizeof (guint32) + 128);
-    bzero (goomInfo->back, buffsize * sizeof (guint32) + 128);
+    memset (goomInfo->back, 0, buffsize * sizeof (guint32) + 128);
     goomInfo->conv = (Pixel *) malloc (buffsize * sizeof (guint32) + 128);
-    bzero (goomInfo->conv, buffsize * sizeof (guint32) + 128);
+    memset (goomInfo->conv, 0, buffsize * sizeof (guint32) + 128);
+
+    goomInfo->outputBuf = goomInfo->conv;
     
     goomInfo->p1 = (Pixel *) ((1 + ((uintptr_t) (goomInfo->pixel)) / 128) * 128);
     goomInfo->p2 = (Pixel *) ((1 + ((uintptr_t) (goomInfo->back)) / 128) * 128);
@@ -64,16 +68,16 @@ PluginInfo *goom_init (guint32 resx, guint32 resy)
     plugin_info_init(goomInfo,4);
     
     goomInfo->star_fx = flying_star_create();
-    goomInfo->star_fx.init(&goomInfo->star_fx);
+    goomInfo->star_fx.init(&goomInfo->star_fx, goomInfo);
     
     goomInfo->zoomFilter_fx = zoomFilterVisualFXWrapper_create ();
-    goomInfo->zoomFilter_fx.init(&goomInfo->zoomFilter_fx);
+    goomInfo->zoomFilter_fx.init(&goomInfo->zoomFilter_fx, goomInfo);
     
     goomInfo->tentacles_fx = tentacle_fx_create();
-    goomInfo->tentacles_fx.init(&goomInfo->tentacles_fx);
+    goomInfo->tentacles_fx.init(&goomInfo->tentacles_fx, goomInfo);
     
     goomInfo->convolve_fx = convolve_create();
-    goomInfo->convolve_fx.init(&goomInfo->convolve_fx);
+    goomInfo->convolve_fx.init(&goomInfo->convolve_fx, goomInfo);
     
     plugin_info_add_visual (goomInfo, 0, &goomInfo->zoomFilter_fx);
     plugin_info_add_visual (goomInfo, 1, &goomInfo->tentacles_fx);
@@ -90,7 +94,7 @@ PluginInfo *goom_init (guint32 resx, guint32 resy)
     goomInfo->cycle = 0;
     
     goomInfo->ifs_fx = ifs_visualfx_create();
-    goomInfo->ifs_fx.init(&goomInfo->ifs_fx);
+    goomInfo->ifs_fx.init(&goomInfo->ifs_fx, goomInfo);
     
     goomInfo->gmline1 = goom_lines_init (goomInfo, resx, goomInfo->screen.height,
                                          GML_HLINE, goomInfo->screen.height, GML_BLACK,
@@ -107,6 +111,7 @@ PluginInfo *goom_init (guint32 resx, guint32 resy)
 }
 
 
+
 void goom_set_resolution (PluginInfo *goomInfo, guint32 resx, guint32 resy)
 {
     free (goomInfo->pixel);
@@ -121,12 +126,18 @@ void goom_set_resolution (PluginInfo *goomInfo, guint32 resx, guint32 resy)
     
     /* init_ifs (goomInfo, resx, goomInfo->screen.height); */
     goomInfo->ifs_fx.free(&goomInfo->ifs_fx);
-    goomInfo->ifs_fx.init(&goomInfo->ifs_fx);
+    goomInfo->ifs_fx.init(&goomInfo->ifs_fx, goomInfo);
     
     goom_lines_set_res (goomInfo->gmline1, resx, goomInfo->screen.height);
     goom_lines_set_res (goomInfo->gmline2, resx, goomInfo->screen.height);
 }
 
+int goom_set_screenbuffer(PluginInfo *goomInfo, void *buffer)
+{
+  goomInfo->outputBuf = (Pixel*)buffer;
+  return 1;
+}
+
 /********************************************
 *                  UPDATE                  *
 ********************************************
@@ -140,7 +151,7 @@ guint32 *goom_update (PluginInfo *goomInfo, gint16 data[2][512],
     guint32 pointWidth;
     guint32 pointHeight;
     int     i;
-    float   largfactor;	/* elargissement de l'intervalle d'volution des points */
+    float   largfactor;	/* elargissement de l'intervalle d'évolution des points */
     Pixel *tmp;
     
     ZoomFilterData *pzfd;
@@ -204,7 +215,7 @@ guint32 *goom_update (PluginInfo *goomInfo, gint16 data[2][512],
         }
     }
     
-    /* par dfaut pas de changement de zoom */
+    /* par défaut pas de changement de zoom */
     pzfd = NULL;
     
     /* 
@@ -230,7 +241,7 @@ guint32 *goom_update (PluginInfo *goomInfo, gint16 data[2][512],
         
         /* changement eventuel de mode */
         if (goom_irand(goomInfo->gRandom,16) == 0)
-            switch (goom_irand(goomInfo->gRandom,32)) {
+            switch (goom_irand(goomInfo->gRandom,34)) {
                 case 0:
                 case 10:
                     goomInfo->update.zoomFilterData.hypercosEffect = goom_irand(goomInfo->gRandom,2);
@@ -298,6 +309,7 @@ guint32 *goom_update (PluginInfo *goomInfo, gint16 data[2][512],
                     break;
                 case 31:
                 case 32:
+                case 33:
                     goomInfo->update.zoomFilterData.mode = SPEEDWAY_MODE;
                     break;
                 default:
@@ -520,7 +532,7 @@ guint32 *goom_update (PluginInfo *goomInfo, gint16 data[2][512],
         }
         
         /*
-         * arreter de decrmenter au bout d'un certain temps
+         * arreter de decrémenter au bout d'un certain temps
          */
         if ((goomInfo->cycle % 101 == 0) && (goomInfo->update.zoomFilterData.pertedec == 7)) {
             pzfd = &goomInfo->update.zoomFilterData;
@@ -588,14 +600,10 @@ guint32 *goom_update (PluginInfo *goomInfo, gint16 data[2][512],
         /*
          * Affichage tentacule
          */
-        
-        /* if (goomInfo.sound.goomlimit > 0.0001f) */
+
         goomInfo->tentacles_fx.apply(&goomInfo->tentacles_fx, goomInfo->p1, goomInfo->p2, goomInfo);
-        /*	tentacle_update(goomInfo, goomInfo->p2, goomInfo->p1, goomInfo->screen.width, goomInfo->screen.height,
-            data, (float)goomInfo->sound.accelvar, goomInfo->curGState->drawTentacle); */
-        
         goomInfo->star_fx.apply (&goomInfo->star_fx,goomInfo->p2,goomInfo->p1,goomInfo);
-        
+ 
         /*
          * Affichage de texte
          */
@@ -730,9 +738,14 @@ guint32 *goom_update (PluginInfo *goomInfo, gint16 data[2][512],
         /* affichage et swappage des buffers.. */
         goomInfo->cycle++;
         
-        goomInfo->convolve_fx.apply(&goomInfo->convolve_fx,return_val,goomInfo->conv,goomInfo);
+        /* xine: no convolve_fx */
+        /*
+        goomInfo->convolve_fx.apply(&goomInfo->convolve_fx,return_val,goomInfo->outputBuf,goomInfo);
+        */
+        xine_fast_memcpy(goomInfo->outputBuf, return_val, goomInfo->screen.size * sizeof(Pixel));
+
         
-        return (guint32*)goomInfo->conv;
+        return (guint32*)goomInfo->outputBuf;
 }
 
 /****************************************
@@ -767,13 +780,13 @@ void goom_close (PluginInfo *goomInfo)
 /* *** */
 void
 choose_a_goom_line (PluginInfo *goomInfo, float *param1, float *param2, int *couleur, int *mode,
-                    float *amplitude, int far)
+                    float *amplitude, int isfar)
 {
     *mode = goom_irand(goomInfo->gRandom,3);
     *amplitude = 1.0f;
     switch (*mode) {
         case GML_CIRCLE:
-            if (far) {
+            if (isfar) {
                 *param1 = *param2 = 0.47f;
                 *amplitude = 0.8f;
                 break;
@@ -791,7 +804,7 @@ choose_a_goom_line (PluginInfo *goomInfo, float *param1, float *param2, int *cou
             }
             break;
         case GML_HLINE:
-            if (goom_irand(goomInfo->gRandom,4) || far) {
+            if (goom_irand(goomInfo->gRandom,4) || isfar) {
                 *param1 = goomInfo->screen.height / 7;
                 *param2 = 6.0f * goomInfo->screen.height / 7.0f;
             }
@@ -801,7 +814,7 @@ choose_a_goom_line (PluginInfo *goomInfo, float *param1, float *param2, int *cou
             }
             break;
         case GML_VLINE:
-            if (goom_irand(goomInfo->gRandom,3) || far) {
+            if (goom_irand(goomInfo->gRandom,3) || isfar) {
                 *param1 = goomInfo->screen.width / 7.0f;
                 *param2 = 6.0f * goomInfo->screen.width / 7.0f;
             }
@@ -828,7 +841,7 @@ void update_message (PluginInfo *goomInfo, char *message) {
     
     if (message) {
         int i=1,j=0;
-        sprintf (goomInfo->update_message.message, message);
+        sprintf (goomInfo->update_message.message, "%s", message);
         for (j=0;goomInfo->update_message.message[j];j++)
             if (goomInfo->update_message.message[j]=='\n')
                 i++;
@@ -843,7 +856,7 @@ void update_message (PluginInfo *goomInfo, char *message) {
         int pos;
         float ecart;
         message = msg;
-        sprintf (msg, goomInfo->update_message.message);
+        sprintf (msg, "%s", goomInfo->update_message.message);
         
         while (!fin) {
             while (1) {
