@@ -23,9 +23,14 @@
 #include "xine_internal.h"
 #include "post.h"
 #include "xineutils.h"
-#include "postprocess.h"
 #include <pthread.h>
 
+#ifdef HAVE_FFMPEG_AVUTIL_H
+#  include <postprocess.h>
+#else
+#  include <libpostproc/postprocess.h>
+#endif
+
 #define PP_STRING_SIZE 256 /* size of pp mode string (including all options) */
 
 /* plugin class initialization function */
