@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  *
  * DirectFB based output plugin by Claudio Ciccani <klan@directfb.org>
@@ -922,7 +922,7 @@ static int directfb_get_property (vo_driver_t *this_gen, int property) {
     
     case VO_PROP_WINDOW_HEIGHT:
       return this->sc.gui_height;
-   
+
     case VO_PROP_MAX_NUM_FRAMES:
       return (this->type & DLTF_VIDEO) ? 8 : 15;
 
@@ -1008,7 +1008,7 @@ static int directfb_set_property (vo_driver_t *this_gen,
         if (value > 0xffffff)
           value = 0xffffff;
         xprintf (this->xine, XINE_VERBOSITY_DEBUG,
-                 "video_out_directfb: setting colorkey to 0x%06x.\n", value);
+                 "video_out_directfb: setting colour key to 0x%06x.\n", value);
         this->colorkey = value;
         this->layer->SetDstColorKey (this->layer, (value & 0xff0000) >> 16,
                                                   (value & 0x00ff00) >>  8,
@@ -1266,7 +1266,7 @@ static void update_config_cb (void *data, xine_cfg_entry_t *entry) {
     if (config.options != this->config.options) {
       if (this->layer->SetConfiguration (this->layer, &config) != DFB_OK) {
           xprintf (this->xine, XINE_VERBOSITY_LOG,
-                   "video_out_directfb: failed to set colorkeying to %d!\n", 
+                   "video_out_directfb: failed to set colour keying to %d!\n", 
                    entry->num_value);
           return;
       }
@@ -1354,15 +1354,15 @@ static void init_config (directfb_driver_t *this) {
   if (this->caps & DLCAPS_DST_COLORKEY) {
     this->colorkeying = config->register_bool  (config,
           "video.device.directfb_colorkeying", this->colorkeying,
-          _("enable video color key"),
-          _("Enable using a color key to tell the graphics card "
+          _("enable video colour key"),
+          _("Enable using a colour key to tell the graphics card "
             "where to overlay the video image."),
           20, update_config_cb, (void *)this);
     
     this->colorkey    = config->register_range (config,
           "video.device.directfb_colorkey", this->colorkey, 0, 0xffffff,
-          _("video color key"),
-          _("The color key is used to tell the graphics card "
+          _("video colour key"),
+          _("The colour key is used to tell the graphics card "
             "where to overlay the video image. Try different values, "
             "if you experience windows becoming transparent."),
           10, update_config_cb, (void  *)this);
@@ -1663,13 +1663,13 @@ static DFBResult init_device (directfb_driver_t *this) {
       
       if (this->visual_type == XINE_VISUAL_TYPE_FB) {
         this->underlay->SetCooperativeLevel (this->underlay, DLSCL_EXCLUSIVE);
-        this->underlay->SetBackgroundMode( this->underlay, DLBM_DONTCARE);
+        this->underlay->SetConfiguration (this->underlay, &config);
       }
     } 
   }
   else {
     /* playing to underlay,
-     * check if stretchblit is hardware accelerated. */
+     * check whether stretchblit is hardware accelerated. */
     IDirectFBSurface      *temp;
     DFBSurfaceDescription  dsc;
     DFBAccelerationMask    mask = DFXL_NONE;
