@@ -15,10 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
- * $Id: video_out_fb.c,v 1.49 2006/12/19 19:10:51 dsalt Exp $
- * 
  * video_out_fb.c, frame buffer xine driver by Miguel Freitas
  *
  * Contributors:
@@ -614,7 +612,19 @@ static int fb_get_property(vo_driver_t *this_gen, int property)
     
     case VO_PROP_WINDOW_HEIGHT:
       return this->sc.gui_height;
-    
+
+    case VO_PROP_OUTPUT_WIDTH:
+      return this->cur_frame->sc.output_width;
+
+    case VO_PROP_OUTPUT_HEIGHT:
+      return this->cur_frame->sc.output_height;
+
+    case VO_PROP_OUTPUT_XOFFSET:
+      return this->cur_frame->sc.output_xoffset;
+
+    case VO_PROP_OUTPUT_YOFFSET:
+      return this->cur_frame->sc.output_yoffset;
+
     default:
       xprintf(this->xine, XINE_VERBOSITY_DEBUG, 
 	      "video_out_fb: tried to get unsupported property %d\n", property);
@@ -757,7 +767,7 @@ static int get_fb_fix_screeninfo(int fd, struct fb_fix_screeninfo *fix, xine_t *
      fix->type != FB_TYPE_PACKED_PIXELS)
   {
     xprintf(xine, XINE_VERBOSITY_LOG,
-	    _("video_out_fb: only packed truecolor/directcolor is supported (%d).\n"
+	    _("video_out_fb: only packed truecolour/directcolour is supported (%d).\n"
 	      "     Check 'fbset -i' or try 'fbset -depth 16'.\n"), fix->visual);
     return 0;
   }
