@@ -15,10 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
- * $Id: video_out_xcbshm.c,v 1.3 2007/03/25 23:13:53 dgp85 Exp $
- * 
  * video_out_xcbshm.c, X11 shared memory extension interface for xine
  *
  * based on mpeg2dec code from
@@ -133,11 +131,6 @@ typedef struct {
  */
 static void create_ximage(xshm_driver_t *this, xshm_frame_t *frame, int width, int height)
 {
-  if (width <= 0)
-    width = 1;
-  if (height <= 0)
-    height = 1;
-
   frame->bytes_per_line = ((this->bpp * width + this->scanline_pad - 1) &
 			   (~(this->scanline_pad - 1))) >> 3;
 
@@ -734,6 +727,14 @@ static int xshm_get_property (vo_driver_t *this_gen, int property) {
     return this->sc.gui_width;
   case VO_PROP_WINDOW_HEIGHT:
     return this->sc.gui_height;
+  case VO_PROP_OUTPUT_WIDTH:
+    return this->cur_frame->sc.output_width;
+  case VO_PROP_OUTPUT_HEIGHT:
+    return this->cur_frame->sc.output_height;
+  case VO_PROP_OUTPUT_XOFFSET:
+    return this->cur_frame->sc.output_xoffset;
+  case VO_PROP_OUTPUT_YOFFSET:
+    return this->cur_frame->sc.output_yoffset;
   default:
     xprintf(this->xine, XINE_VERBOSITY_DEBUG, 
 	    "video_out_xcbshm: tried to get unsupported property %d\n", property);
