@@ -15,10 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
- * $Id: video_out_xvmc.c,v 1.29 2007/03/25 23:07:23 dgp85 Exp $
- * 
  * video_out_xvmc.c, X11 video motion compensation extension interface for xine
  *
  * based on mpeg2dec code from
@@ -766,6 +764,11 @@ static void xvmc_update_frame_format (vo_driver_t *this_gen,
   xvmc_frame_t   *frame = (xvmc_frame_t *) frame_gen;
   xine_xvmc_t *xvmc = (xine_xvmc_t *) frame_gen->accel_data;
 
+  if (format != XINE_IMGFMT_XVMC) {
+    xprintf (this->xine, XINE_VERBOSITY_LOG, "xvmc_update_frame_format: frame format %08x not supported\n", format);
+    _x_abort();
+  }
+
   lprintf ("xvmc_update_frame_format\n");
 
   if ((frame->width != width)
@@ -853,6 +856,8 @@ static void xvmc_overlay_blend (vo_driver_t *this_gen,
       _x_blend_yuy2(frame->vo_frame.base[0], overlay, 
 		 frame->width, frame->height, frame->vo_frame.pitches[0],
                  &this->alphablend_extra_data);
+    else
+      xprintf (this->xine, XINE_VERBOSITY_LOG, "xvmc_overlay_blend: overlay blending not supported for frame format %08x\n", frame->format);
   }
 }
 
@@ -1004,6 +1009,18 @@ static int xvmc_get_property (vo_driver_t *this_gen, int property) {
     case VO_PROP_WINDOW_HEIGHT:
       this->props[property].value = this->sc.gui_height;
       break;
+    case VO_PROP_OUTPUT_WIDTH:
+      this->props[property].value = this->sc.output_width;
+      break;
+    case VO_PROP_OUTPUT_HEIGHT:
+      this->props[property].value = this->sc.output_height;
+      break;
+    case VO_PROP_OUTPUT_XOFFSET:
+      this->props[property].value = this->sc.output_xoffset;
+      break;
+    case VO_PROP_OUTPUT_YOFFSET:
+      this->props[property].value = this->sc.output_yoffset;
+      break;
   }
   
   return this->props[property].value;
@@ -1453,7 +1470,7 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *vi
 				 class->adaptor_info[class->adaptor_num].base_id, "XV_AUTOPAINT_COLORKEY",
 				 "video.device.xv_autopaint_colorkey",
 				 _("autopaint colour key"),
-				 _("Make Xv autopaint its colorkey."));
+				 _("Make Xv autopaint its colour key."));
 
 	} else if(!strcmp(attr[k].name, "XV_DOUBLE_BUFFER")) {
 	  int xvmc_double_buffer;
@@ -1744,7 +1761,7 @@ static void *init_class (xine_t *xine, void *visual_gen) {
   else {
     xprintf (xine, XINE_VERBOSITY_LOG, 
 	     _("video_out_xvmc: using Xv port %ld from adaptor %s\n"
-	       "                for hardware colorspace conversion and scaling\n"),
+	       "                for hardware colour space conversion and scaling\n"),
 	     xv_port, adaptor_info[adaptor_num].name);
     
     if(IDCTaccel&XINE_VO_IDCT_ACCEL)
