@@ -16,9 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: video_out_xxmc.c,v 1.23 2007/03/25 23:13:53 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * video_out_xxmc.c, X11 decoding accelerated video extension interface for xine
  *
@@ -33,7 +31,6 @@
  * X11 unscaled overlay support by Miguel Freitas - Nov 2003
  * XvMC VLD implementation by Thomas Hellström - 2004, 2005.
  * XvMC merge by Thomas Hellström - Sep 2004
- *
  */
 
 
@@ -608,11 +605,6 @@ static XvImage *create_ximage (xxmc_driver_t *this, XShmSegmentInfo *shminfo,
   unsigned int  xv_format;
   XvImage      *image = NULL;
 
-  if (width <= 0)
-    width = 1;
-  if (height <= 0)
-    height = 1;
-
   if (this->use_pitch_alignment) {
     width = (width + 7) & ~0x7;
   }
@@ -1768,6 +1760,18 @@ static int xxmc_get_property (vo_driver_t *this_gen, int property) {
   case VO_PROP_WINDOW_HEIGHT:
     this->props[property].value = this->sc.gui_height;
     break;
+  case VO_PROP_OUTPUT_WIDTH:
+    this->props[property].value = this->sc.output_width;
+    break;
+  case VO_PROP_OUTPUT_HEIGHT:
+    this->props[property].value = this->sc.output_height;
+    break;
+  case VO_PROP_OUTPUT_XOFFSET:
+    this->props[property].value = this->sc.output_xoffset;
+    break;
+  case VO_PROP_OUTPUT_YOFFSET:
+    this->props[property].value = this->sc.output_yoffset;
+    break;
   }
 
   lprintf("video_out_xxmc: property #%d = %d\n", property, this->props[property].value);
@@ -2432,7 +2436,7 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *vi
   else
     xprintf(class->xine, XINE_VERBOSITY_LOG,
 	    _("video_out_xxmc: using Xv port %ld from adaptor %s for hardware "
-	      "colorspace conversion and scaling.\n"), xv_port,
+	      "colour space conversion and scaling.\n"), xv_port,
             adaptor_info[adaptor_num].name);
   
   XUnlockDisplay(this->display);
@@ -2543,7 +2547,7 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *vi
 				 adaptor_info[adaptor_num].base_id, "XV_AUTOPAINT_COLORKEY",
 				 "video.device.xv_autopaint_colorkey",
 				 _("autopaint colour key"),
-				 _("Make Xv autopaint its colorkey."));
+				 _("Make Xv autopaint its colour key."));
 
 	} else if(!strcmp(attr[k].name, "XV_FILTER")) {
 	  int xv_filter;
@@ -2655,8 +2659,8 @@ static vo_driver_t *open_plugin (video_driver_class_t *class_gen, const void *vi
 			   10, xxmc_update_cpu_save, this);
   this->reverse_nvidia_palette =
     config->register_bool (config, "video.device.xvmc_nvidia_color_fix", 0,
-			   _("Fix buggy NVIDIA XvMC subpicture colors"),
-			   _("There's a bug in NVIDIA's XvMC lib that makes red OSD colors\n"
+			   _("Fix buggy NVIDIA XvMC subpicture colours"),
+			   _("There's a bug in NVIDIA's XvMC lib that makes red OSD colours\n"
 			     "look blue and vice versa. This option provides a workaround.\n"),
 			   10, xxmc_update_nvidia_fix, this);
   this->bob =
