@@ -21,9 +21,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * $Id: yuv2rgb.c,v 1.52 2006/06/20 01:07:59 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 #include "config.h"
@@ -3222,7 +3220,7 @@ yuv2rgb_factory_t* yuv2rgb_factory_init (int mode, int swapped,
 
 #ifdef LOG
     if (this->yuv2rgb_fun != NULL)
-      printf ("yuv2rgb: using MMXEXT for colorspace transform\n");
+      printf ("yuv2rgb: using MMXEXT for colour space transform\n");
 #endif
   }
 
@@ -3232,7 +3230,7 @@ yuv2rgb_factory_t* yuv2rgb_factory_init (int mode, int swapped,
 
 #ifdef LOG
     if (this->yuv2rgb_fun != NULL)
-      printf ("yuv2rgb: using MMX for colorspace transform\n");
+      printf ("yuv2rgb: using MMX for colour space transform\n");
 #endif
   }
 #endif
@@ -3243,12 +3241,12 @@ yuv2rgb_factory_t* yuv2rgb_factory_init (int mode, int swapped,
 
 #ifdef LOG
     if (this->yuv2rgb_fun != NULL)
-      printf ("yuv2rgb: using medialib for colorspace transform\n");
+      printf ("yuv2rgb: using medialib for colour space transform\n");
 #endif
   }
 #endif
   if (this->yuv2rgb_fun == NULL) {
-    lprintf ("no accelerated colorspace conversion found\n");
+    lprintf ("no accelerated colour space conversion found\n");
 
     yuv2rgb_c_init (this);
   }
