@@ -15,12 +15,9 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: configfile.c,v 1.85 2007/02/22 15:49:16 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * config object (was: file) management - implementation
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1063,7 +1060,7 @@ void xine_config_save (xine_t *xine, const char *filename) {
 
       if (!entry->key[0])
         /* deleted key */
-        continue;
+        goto next;
 
       lprintf ("saving key '%s'\n", entry->key);
 
@@ -1133,6 +1130,7 @@ void xine_config_save (xine_t *xine, const char *filename) {
 	break;
       }
 
+      next:
       entry = entry->next;
     }
     pthread_mutex_unlock(&this->config_lock);
