@@ -15,13 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: xine_interface.c,v 1.104 2007/03/26 11:48:01 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * convenience/abstraction layer, functions to implement
  * libxine's public interface
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,7 +32,7 @@
 #include <stdlib.h>
 #include <pthread.h>
 #include <stdarg.h>
-#if defined (__linux__)
+#if defined (__linux__) || defined (__GLIBC__)
 #include <endian.h>
 #elif defined (__FreeBSD__)
 #include <machine/endian.h>
