@@ -15,11 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with the Gnome Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- *
- *  $Id: xmllexer.c,v 1.13 2007/03/04 16:19:12 hadess Exp $
- *
+ * write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
+ * Floor, Boston, MA 02110, USA
  */
 
 #define LOG_MODULE "xmllexer"
@@ -39,6 +36,11 @@
 #include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
+#ifdef HAVE_ICONV
+#include <iconv.h>
+#endif
+
+#include "bswap.h"
 
 /* private constants*/
 #define NORMAL       0  /* normal lex mode */
@@ -50,10 +52,70 @@ static int lexbuf_size = 0;
 static int lexbuf_pos  = 0;
 static int lex_mode    = NORMAL;
 static int in_comment  = 0;
+static char *lex_malloc = NULL;
+
+enum utf { UTF32BE, UTF32LE, UTF16BE, UTF16LE };
+
+static void lex_convert (const char * buf, int size, enum utf utf)
+{
+  char *utf8 = malloc (size * (utf >= UTF16BE ? 3 : 6) + 1);
+  char *bp = utf8;
+  while (size > 0)
+  {
+    uint32_t c = 0;
+    switch (utf)
+    {
+    case UTF32BE: c = _X_BE_32 (buf); buf += 4; break;
+    case UTF32LE: c = _X_LE_32 (buf); buf += 4; break;
+    case UTF16BE: c = _X_BE_16 (buf); buf += 2; break;
+    case UTF16LE: c = _X_LE_16 (buf); buf += 2; break;
+    }
+    if (!c)
+      break; /* embed a NUL, get a truncated string */
+    if (c < 128)
+      *bp++ = c;
+    else
+    {
+      int count = (c >= 0x04000000) ? 5 :
+		  (c >= 0x00200000) ? 4 :
+		  (c >= 0x00010000) ? 3 :
+		  (c >= 0x00000800) ? 2 : 1;
+      *bp = (char)(0x1F80 >> count);
+      count *= 6;
+      *bp++ |= c >> count;
+      while ((count -= 6) >= 0)
+	*bp++ = 128 | ((c >> count) & 0x3F);
+    }
+  }
+  *bp = 0;
+  lexbuf_size = bp - utf8;
+  lexbuf = lex_malloc = realloc (utf8, lexbuf_size + 1);
+}
 
 void lexer_init(const char * buf, int size) {
+  static const char boms[] = { 0xFF, 0xFE, 0, 0, 0xFE, 0xFF },
+		    bom_utf8[] = { 0xEF, 0xBB, 0xBF };
+
+  free (lex_malloc);
+  lex_malloc = NULL;
+
   lexbuf      = buf;
   lexbuf_size = size;
+
+  if (size >= 4 && !memcmp (buf, boms + 2, 4))
+    lex_convert (buf + 4, size - 4, UTF32BE);
+  else if (size >= 4 && !memcmp (buf, boms, 4))
+    lex_convert (buf + 4, size - 4, UTF32LE);
+  else if (size >= 3 && !memcmp (buf, bom_utf8, 3))
+  {
+    lexbuf += 3;
+    lexbuf_size -= 3;
+  }
+  else if (size >= 2 && !memcmp (buf, boms + 4, 2))
+    lex_convert (buf + 2, size - 2, UTF16BE);
+  else if (size >= 2 && !memcmp (buf, boms, 2))
+    lex_convert (buf + 2, size - 2, UTF16LE);
+
   lexbuf_pos  = 0;
   lex_mode    = NORMAL;
   in_comment  = 0;
@@ -61,7 +123,10 @@ void lexer_init(const char * buf, int size) {
   lprintf("buffer length %d\n", size);
 }
 
-int lexer_get_token(char * tok, int tok_size) {
+int lexer_get_token_d(char ** _tok, int * _tok_size, int fixed) {
+  char *tok = *_tok;
+  int tok_size = *_tok_size;
+
   int tok_pos = 0;
   int state = 0;
   char c;
@@ -346,6 +411,8 @@ int lexer_get_token(char * tok, int tok_size) {
 	  case '\"': /* " */
 	  case ' ':
 	  case '\t':
+	  case '\n':
+	  case '\r':
 	  case '=':
 	  case '/':
 	    tok[tok_pos] = '\0';
@@ -393,7 +460,17 @@ int lexer_get_token(char * tok, int tok_size) {
 
     /* pb */
     if (tok_pos >= tok_size) {
-      lprintf("token buffer is too little\n");
+      if (fixed)
+        return T_ERROR;
+      *_tok_size *= 2;
+      *_tok = realloc (*_tok, *_tok_size);
+      lprintf("token buffer is too small\n");
+      lprintf("increasing buffer size to %d bytes\n", *_tok_size);
+      if (*_tok) {
+          return lexer_get_token_d (_tok, _tok_size, 0);
+      } else {
+          return T_ERROR;
+      }
     } else {
       if (lexbuf_pos >= lexbuf_size) {
 				/* Terminate the current token */
@@ -436,6 +513,12 @@ int lexer_get_token(char * tok, int tok_size) {
   return T_ERROR;
 }
 
+/* for ABI compatibility */
+int lexer_get_token (char *tok, int tok_size)
+{
+  return lexer_get_token_d (&tok, &tok_size, 1);
+}
+
 static struct {
   char code;
   unsigned char namelen;
@@ -463,7 +546,7 @@ char *lexer_decode_entities (const char *tok)
     {
       /* parse the character entity (on failure, treat it as literal text) */
       const char *tp = tok;
-      long i;
+      signed long i;
 
       for (i = 0; lexer_entities[i].code; ++i)
 	if (!strncmp (lexer_entities[i].name, tok, lexer_entities[i].namelen)
@@ -491,7 +574,7 @@ char *lexer_decode_entities (const char *tok)
       else
 	i = strtol (tp, (char **)&tp, 10);
 
-      if (i < 1 || i > 255 || *tp != ';')
+      if (*tp != ';' || i < 1)
       {
         /* out of range, or format error */
 	*bp++ = '&';
@@ -499,7 +582,23 @@ char *lexer_decode_entities (const char *tok)
       }
 
       tok = tp + 1;
-      *bp++ = i;
+
+      if (i < 128)
+        /* ASCII - store as-is */
+	*bp++ = i;
+      else
+      {
+	/* Non-ASCII, so convert to UTF-8 */
+	int count = (i >= 0x04000000) ? 5 :
+		    (i >= 0x00200000) ? 4 :
+		    (i >= 0x00010000) ? 3 :
+		    (i >= 0x00000800) ? 2 : 1;
+	*bp = (char)(0x1F80 >> count);
+	count *= 6;
+	*bp++ |= i >> count;
+	while ((count -= 6) >= 0)
+	  *bp++ = 128 | ((i >> count) & 0x3F);
+      }
     }
   }
   *bp = 0;
